from collections import defaultdict
from concurrent.futures import ThreadPoolExecutor, as_completed
from ipaddress import ip_network

from ezsnmp import EzSNMPError, Session
from icmplib import ping
from tcppinglib import tcpping

from netty_snmp._types import (
    DeviceType,
    DiscoveryData,
    DiscoveryException,
    DiscoveryItem,
    DiscoveryResponse,
    DispatchItem,
    IPvANyNetwork,
)
from netty_snmp.device_type.device_types import Platform, get_device_type
from netty_snmp.factory import consts
from netty_snmp.factory.manufactures.arista import AristaSnmpFactory
from netty_snmp.factory.manufactures.aruba import ArubaSnmpFactory
from netty_snmp.factory.manufactures.cisco import CiscoSnmpFactory
from netty_snmp.factory.manufactures.fortinet import FortinetSnmpFactory
from netty_snmp.factory.manufactures.h3c import H3cSnmpFactory
from netty_snmp.factory.manufactures.huawei import HuaweiSnmpFactory
from netty_snmp.factory.manufactures.juniper import JuniperSnmpFactory
from netty_snmp.factory.manufactures.paloalto import PaloAltoSnmpFactory
from netty_snmp.factory.manufactures.ruijie import RuijieSnmpFactory
from netty_snmp.factory.snmp_factory import SnmpFactory, SnmpV3Params


def get_factory(platform: Platform) -> type[SnmpFactory]:
    factories = {
        Platform.Arista: AristaSnmpFactory,
        Platform.CiscoIOS: CiscoSnmpFactory,
        Platform.CiscoIOSXE: CiscoSnmpFactory,
        Platform.CiscoIOSXR: CiscoSnmpFactory,
        Platform.CiscoNexusOS: CiscoSnmpFactory,
        Platform.FortiNet: FortinetSnmpFactory,
        Platform.H3C: H3cSnmpFactory,
        Platform.Huawei: HuaweiSnmpFactory,
        Platform.HuaweiVRP: HuaweiSnmpFactory,
        Platform.HuaweiVRPV8: HuaweiSnmpFactory,
        Platform.Arista: AristaSnmpFactory,
        Platform.Aruba: ArubaSnmpFactory,
        Platform.RuiJie: RuijieSnmpFactory,
        Platform.Juniper: JuniperSnmpFactory,
        Platform.PaloAlto: PaloAltoSnmpFactory,
    }

    return factories.get(platform, SnmpFactory)


class DispatchSnmpFactory:
    def __init__(
        self,
        prefix: str,
        port: int = consts.SNMP_DEFAULT_PORT,
        version: consts.SnmpVersion = consts.SnmpVersion.v2c,
        community: str | None = consts.SNMP_DEFAULT_COMMUNITY,
        v3_params: SnmpV3Params | None = None,
        snmp_max_repetitions: int = 20,
        max_workers: int = 16,
    ) -> None:
        self.prefix = self.str_to_prefix(prefix)
        self.port = port
        self.version = version
        self.community = community
        self.v3_params = v3_params
        self.max_workers = max_workers
        self.exceptions: dict[str, list[DiscoveryException]] = defaultdict(list)
        self.snmp_max_repetitions = snmp_max_repetitions

    def str_to_prefix(self, prefix: str) -> IPvANyNetwork:
        if "/" not in prefix:
            if ":" in prefix or "::" in prefix:
                prefix += "/128"
            else:
                prefix += "/32"
        try:
            return ip_network(prefix)
        except ValueError as e:
            raise ValueError(f"Invalid ip prefix: {prefix}") from e

    def snmp_reachable(self, session: Session) -> bool:
        try:
            result = session.get_next(".1").value
        except EzSNMPError:
            result = None
        return result is not None

    def sys_object_id(self, session: Session) -> str | None:
        return session.get(consts.sysObjectID.oid).value

    def device_type(self, sys_object_id: str) -> "DeviceType":
        device_type = get_device_type(sys_object_id)
        if not device_type:
            return DeviceType(
                platform=consts.UNKNOWN_PLATFORM,
                manufacturer=consts.UNKNOWN_MANUFACTURER,
                model=consts.UNKNOWN_MODEL,
            )
        return device_type

    def get_snmp_session(self, ip: str) -> Session:
        if self.version == consts.SnmpVersion.v2c and self.community:
            session = Session(
                hostname=ip, remote_port=self.port, community=self.community, version=consts.SnmpVersion.v2c.value
            )
        elif self.version == consts.SnmpVersion.v3 and self.v3_params:
            session = Session(hostname=ip, remote_port=self.port, version=consts.SnmpVersion.v3.value, **self.v3_params)
        else:
            raise ValueError("Unsupported SNMP version")
        return session

    def _dispatch(self, ip_address: str, discovery_items: list[DiscoveryItem] | None = None) -> DiscoveryResponse:
        snmp_session = self.get_snmp_session(ip_address)
        icmp_reachable = self._is_icmp_reachable(ip_address)
        ssh_reachable = self._is_ssh_reachable(ip_address)
        snmp_reachable = self.snmp_reachable(snmp_session)

        discovery_response = DiscoveryResponse(
            ip=ip_address,
            data=None,
            snmp_reachable=snmp_reachable,
            icmp_reachable=icmp_reachable,
            ssh_reachable=ssh_reachable,
            sys_object_id=None,
            exceptions=None,
        )

        if not snmp_reachable:
            return discovery_response

        try:
            sys_object_id = self.sys_object_id(snmp_session)
        except EzSNMPError as e:
            self._handle_snmp_error(ip_address, "sys_object_id", e)
            return discovery_response

        if not sys_object_id:
            return discovery_response

        device_type = self.device_type(sys_object_id)
        factory = get_factory(device_type["platform"])  # type: ignore  # noqa: PGH003
        device_data = factory(
            ip=ip_address,
            port=self.port,
            version=self.version,
            community=self.community,
            v3_params=self.v3_params,
            snmp_max_repetitions=self.snmp_max_repetitions,
        ).discovery(discovery_items)

        discovery_data = DiscoveryData(
            device_type=device_type["model"],
            manufacturer=device_type["manufacturer"],
            platform=device_type["platform"],
            **device_data,
        )

        discovery_response["data"] = discovery_data
        discovery_response["sys_object_id"] = sys_object_id

        return discovery_response

    @staticmethod
    def _is_icmp_reachable(ip_address: str) -> bool:
        return ping(ip_address, count=2, interval=0.2, timeout=1, privileged=False).is_alive  # type: ignore  # noqa: PGH003

    @staticmethod
    def _is_ssh_reachable(ip_address: str) -> bool:
        return tcpping(ip_address, port=22, timeout=1, count=2, interval=0.2).is_alive

    def _handle_snmp_error(self, ip_address: str, item: DispatchItem, exception: EzSNMPError) -> None:
        self.exceptions.setdefault(ip_address, []).append(DiscoveryException(item=item, exception=str(exception)))
        self.exceptions[ip_address].append(DiscoveryException(item=item, exception=str(exception)))

    def dispatch(self, discovery_items: list[DiscoveryItem] | None = None) -> list[DiscoveryResponse]:
        with ThreadPoolExecutor(max_workers=self.max_workers) as executor:
            futures = [executor.submit(self._dispatch, str(ip), discovery_items) for ip in self.prefix.hosts()]
            return [future.result() for future in as_completed(futures)]



from netty_snmp.factory import consts
from netty_snmp.factory.snmp_factory import SnmpFactory, SnmpV3Params


class JuniperSnmpFactory(SnmpFactory):
    def __init__(
        self,
        ip: str,
        port: int = consts.SNMP_DEFAULT_PORT,
        version: consts.SnmpVersion = consts.SnmpVersion.v2c,
        community: str | None = consts.SNMP_DEFAULT_COMMUNITY,
        v3_params: SnmpV3Params | None = None,
        model: str | None = None,
    ) -> None:
        super().__init__(ip, port, version, community, v3_params, model)



from ezsnmp import EzSNMPError

from netty_snmp._types import DiscoveryException, StackMember
from netty_snmp.factory import consts
from netty_snmp.factory.snmp_factory import SnmpFactory, SnmpV3Params
from netty_snmp.utils import mac_address_validator


class HuaweiSnmpFactory(SnmpFactory):
    def __init__(
        self,
        ip: str,
        port: int = consts.SNMP_DEFAULT_PORT,
        version: consts.SnmpVersion = consts.SnmpVersion.v2c,
        community: str | None = consts.SNMP_DEFAULT_COMMUNITY,
        v3_params: SnmpV3Params | None = None,
    ) -> None:
        super().__init__(ip, port, version, community, v3_params)

    @property
    def stack(self) -> list[StackMember]:
        stack_running = self.session.get(consts.hwStackRun.oid).value
        if stack_running != "1":
            return []
        try:
            hw_stack_id = self.session.bulkwalk(
                consts.hwMemberCurrentStackId.oid, max_repetitions=self.snmp_max_repetitions
            )
            hw_stack_priority = self.session.bulkwalk(
                consts.hwMemberStackPriority.oid, max_repetitions=self.snmp_max_repetitions
            )
            hw_stack_role = self.session.bulkwalk(
                consts.hwMemberStackRole.oid, max_repetitions=self.snmp_max_repetitions
            )
            hw_stack_mac_address = self.session.bulkwalk(
                consts.hwMemberStackMacAddress.oid, max_repetitions=self.snmp_max_repetitions
            )

        except EzSNMPError as e:
            self.exceptions.append(DiscoveryException(item="stack", exception=str(e)))
            return []
        index_hw_stack_id = {x.oid_index: x.value for x in hw_stack_id}
        index_hw_stack_priority = {x.oid_index: x.value for x in hw_stack_priority}
        index_hw_stack_role = {x.oid_index: x.value for x in hw_stack_role}
        index_hw_stack_mac_address = {x.oid_index: mac_address_validator(x.value) for x in hw_stack_mac_address}
        return [
            StackMember(
                id=index_hw_stack_id.get(x),
                priority=index_hw_stack_priority.get(x),
                role=index_hw_stack_role.get(x),
                mac_address=index_hw_stack_mac_address.get(x),
            )
            for x in index_hw_stack_id
        ]



import re
from typing import overload

MAC_ADDRESS_LENGTH = 12


@overload
def mac_address_validator(mac: None) -> None: ...


@overload
def mac_address_validator(mac: str) -> str: ...


def mac_address_validator(mac: str | None) -> str | None:
    """
    Validates the given MAC address. if given MAC address is not valid, it will be returned as it is.

    Args:
        mac (str): The MAC address to be validated.

    Returns:
        str: The validated MAC address in colon-separated format.
    """
    if mac is None:
        return mac
    _mac_address_re = r"^[0-9a-fA-F]{12}$"
    input_mac = re.sub(r"[^0-9a-fA-F]", "", mac)
    if len(input_mac) != MAC_ADDRESS_LENGTH:
        return mac
    if re.match(_mac_address_re, input_mac):
        return ":".join(input_mac[i : i + 2] for i in range(0, len(input_mac), 2)).lower()
    return mac



from netty_snmp import DispatchSnmpFactory, consts


def test_dispatch():
    factory = DispatchSnmpFactory(
        prefix="192.168.1.0/26",
        port=161,
        version=consts.SnmpVersion.v2c,
        community="public",
        snmp_max_repetitions=20,
        max_workers=64,
    )

    result = factory.dispatch()
    assert result

    result1 = factory.dispatch(discovery_items=["hostname"])

    assert result1

    result2 = factory.dispatch(discovery_items=["hostname", "uptime"])
    assert result2



from enum import IntEnum

from netty_snmp._types import SnmpItem

SNMP_DEFAULT_PORT = 161
SNMP_DEFAULT_COMMUNITY = "public"
SNMP_MAX_repetitions = 50
UNKNOWN_MANUFACTURER = "Unknown"
UNKNOWN_PLATFORM = "Unknown"
UNKNOWN_MODEL = "Unknown"
UNKNOWN_PORT_MODE = "Unknown"


class SnmpVersion(IntEnum):
    v1 = 1
    v2c = 2
    v3 = 3


class StackRole(IntEnum):
    Master = 1
    Member = 2
    NotMember = 3
    StandBy = 4


INTERFACE_STATUS_MAPPING = {1: "up", 2: "down", 3: "testing"}

LLDP_CHASSIS_ID_SUBTYPE_MAPPING = {
    1: "Chassis Component",
    2: "Interface Alias",
    3: "Port Component",
    4: "MAC Address",
    5: "Network Address",
    6: "Interface Name",
    7: "Local",
}

LLDP_PORT_ID_SUBTYPE_MAPPING = {
    1: "Interface Alias",
    2: "Port Component",
    3: "MAC Address",
    4: "Network Address",
    5: "Interface Name",
    6: "Agent Circuit ID",
    7: "Local",
}

ENTITY_PHYSICAL_CLASS_MAPPING = {
    1: "Other",
    2: "Unknown",
    3: "Chassis",
    4: "Backplane",
    5: "Container",
    6: "PowerSupply",
    7: "Fan",
    8: "Sensor",
    9: "Module",
    10: "Port",
    11: "Stack",
    12: "CPU",
}

HW_STACK_RUN_MAPPING = {1: "enable", 2: "disable"}
HW_STACK_ROLE_MAPPING = {1: "master", 2: "backup", 3: "slave"}


# ---------- system ---------- #

sysDescr = SnmpItem(
    name="sysDescr",
    oid=".1.3.6.1.2.1.1.1.0",
    description="The textual description of the network management subsystem.",
    value_type="str",
    value_mapping=None,
)

sysObjectID = SnmpItem(
    name="sysObjectID",
    oid=".1.3.6.1.2.1.1.2.0",
    description="The vendor's authoritative identification of the network management subsystem contained in the entity.",
    value_type="str",
    value_mapping=None,
)

sysUpTime = SnmpItem(
    name="sysUpTime",
    oid=".1.3.6.1.2.1.1.3.0",
    description="The time that the network management subsystem is running.",
    value_type="int",
    value_mapping=None,
)

sysName = SnmpItem(
    name="sysName",
    oid=".1.3.6.1.2.1.1.5.0",
    description="The name of the network management subsystem.",
    value_type="str",
    value_mapping=None,
)

# ---------- interface ------------- #
ifIndex = SnmpItem(
    name="ifIndex",
    oid=".1.3.6.1.2.1.2.2.1.1",
    description="The ifIndex value for the interface.",
    value_type="int",
    value_mapping=None,
)

ifDescr = SnmpItem(
    name="ifDescr",
    oid=".1.3.6.1.2.1.2.2.1.2",
    description="The ifDescr value for the interface.",
    value_type="str",
    value_mapping=None,
)

ifType = SnmpItem(
    name="ifType",
    oid=".1.3.6.1.2.1.2.2.1.3",
    description="The ifType value for the interface.",
    value_type="int",
    value_mapping=None,
)

ifMtu = SnmpItem(
    name="ifMtu",
    oid=".1.3.6.1.2.1.2.2.1.4",
    description="The ifMtu value for the interface.",
    value_type="int",
    value_mapping=None,
)

ifSpeed = SnmpItem(
    name="ifSpeed",
    oid=".1.3.6.1.2.1.2.2.1.5",
    description="The ifSpeed value for the interface.",
    value_type="int",
    value_mapping=None,
)

ifPhysAddr = SnmpItem(
    name="ifPhysAddr",
    oid=".1.3.6.1.2.1.2.2.1.6",
    description="The ifPhysAddr value for the interface.",
    value_type="str",
    value_mapping=None,
)

ifAdminStatus = SnmpItem(
    name="ifAdminStatus",
    oid=".1.3.6.1.2.1.2.2.1.7",
    description="The ifAdminStatus value for the interface.",
    value_type="int",
    value_mapping={
        1: "up",
        2: "down",
        3: "testing",
    },
)

ifOperStatus = SnmpItem(
    name="ifOperStatus",
    oid=".1.3.6.1.2.1.2.2.1.8",
    description="The ifOperStatus value for the interface.",
    value_type="int",
    value_mapping={
        1: "up",
        2: "down",
        3: "testing",
    },
)

ifLastChange = SnmpItem(
    name="ifLastChange",
    oid=".1.3.6.1.2.1.2.2.1.9",
    description="The ifLastChange value for the interface.",
    value_type="int",
    value_mapping=None,
)

ifAlias = SnmpItem(
    name="ifAlias",
    oid=".1.3.6.1.2.1.31.1.1.1.18.0",
    description="The ifAlias value for the interface.",
    value_type="str",
    value_mapping=None,
)
ifHighSpeed = SnmpItem(
    name="ifHighSpeed",
    oid=".1.3.6.1.2.1.31.1.1.1.15",
    description="The ifHighSpeed value for the interface.",
    value_type="int",
    value_mapping=None,
)
ifAdEntIfIndex = SnmpItem(
    name="ifAdEntIfIndex",
    oid="1.3.6.1.2.1.4.20.1.2",
    description="The ifAdEntIfIndex value for the interface.",
    value_type="int",
    value_mapping=None,
)
ifAdEntNetMask = SnmpItem(
    name="ifAdEntNetMask",
    oid="1.3.6.1.2.1.4.20.1.3",
    description="The ifAdEntNetMask value for the interface.",
    value_type="str",
    value_mapping=None,
)


# ---------- lldp ---------- #

lldpLocChassisId = SnmpItem(
    name="lldpLocChassisId",
    oid=".1.0.8802.1.1.2.1.3.2.0",
    description="The lldpLocChassisId value for the interface.",
    value_type="str",
    value_mapping=None,
    to_hex=True,
)

lldpLoPortId = SnmpItem(
    name="lldpLocPortId",
    oid=".1.0.8802.1.1.2.1.3.7.1.3",
    description="The lldpLocalPortId value for the interface.",
    value_type="str",
    value_mapping=None,
    to_hex=True,
)

lldpLocPortDesc = SnmpItem(
    name="lldpLocPortDesc",
    oid=".1.0.8802.1.1.2.1.3.7.1.4",
    description="The lldpLocPortDesc value for the interface.",
    value_type="str",
    value_mapping=None,
)

lldpRemChassisIdSubtype = SnmpItem(
    name="lldpRemChassisIdSubtype",
    oid=".1.0.8802.1.1.2.1.4.1.1.4",
    description="The lldpRemChassisIdSubtype value for the interface.",
    value_type="int",
    value_mapping=LLDP_CHASSIS_ID_SUBTYPE_MAPPING,
)

lldpRemChassisId = SnmpItem(
    name="lldpRemChassisId",
    oid=".1.0.8802.1.1.2.1.4.1.1.5",
    description="The lldpRemChassisId value for the interface.",
    value_type="str",
    value_mapping=None,
    to_hex=True,
)


lldpRemPortIdSubtype = SnmpItem(
    name="lldpRemPortIdSubtype",
    oid=".1.0.8802.1.1.2.1.4.1.1.6",
    description="The lldpRemPortIdSubtype value for the interface.",
    value_type="int",
    value_mapping=None,
)

lldpRemPortId = SnmpItem(
    name="lldpRemPortId",
    oid=".1.0.8802.1.1.2.1.4.1.1.7",
    description="The lldpRemPortId value for the interface.",
    value_type="str",
    value_mapping=None,
    to_hex=False,
)

lldpRemPortDesc = SnmpItem(
    name="lldpRemPortDesc",
    oid=".1.0.8802.1.1.2.1.4.1.1.8",
    description="The lldpRemPortDesc value for the interface.",
    value_type="str",
    value_mapping=None,
)

lldpRemSysName = SnmpItem(
    name="lldpRemSysName",
    oid=".1.0.8802.1.1.2.1.4.1.1.9",
    description="The lldpRemSysName value for the interface.",
    value_type="str",
    value_mapping=None,
)


# ---------- entity ---------- #

entPhysicalDescr = SnmpItem(
    name="entPhysicalDescr",
    oid=".1.3.6.1.2.1.47.1.1.1.1.2",
    description="A textual description of physical entity.",
    value_type="str",
    value_mapping=None,
)

entPhysicalClass = SnmpItem(
    name="entPhysicalClass",
    oid=".1.3.6.1.2.1.47.1.1.1.1.5",
    description="An indication of the general hardware type of the physical entity.",
    value_type="int",
    value_mapping=ENTITY_PHYSICAL_CLASS_MAPPING,
)
entPhysicalName = SnmpItem(
    name="entPhysicalName",
    oid=".1.3.6.1.2.1.47.1.1.1.1.7",
    description="The textual name of the physical entity.",
    value_type="str",
    value_mapping=None,
)

entPhysicalHardwareRev = SnmpItem(
    name="entPhysicalHardwareRev",
    oid=".1.3.6.1.2.1.47.1.1.1.1.8",
    description="The hardware revision of the physical entity.",
    value_type="str",
    value_mapping=None,
)  # not include in discovery
entPhysicalFirmwareRev = SnmpItem(
    name="entPhysicalFirmwareRev",
    oid=".1.3.6.1.2.1.47.1.1.1.1.9",
    description="The firmware revision of the physical entity.",
    value_type="str",
    value_mapping=None,
)  # not include in discovery

entPhysicalSoftwareRev = SnmpItem(
    name="entPhysicalSoftwareRev",
    oid=".1.3.6.1.2.1.47.1.1.1.1.10",
    description="The software revision of the physical entity.",
    value_type="str",
    value_mapping=None,
)

entPhysicalSerialNum = SnmpItem(
    name="entPhysicalSerialNum",
    oid=".1.3.6.1.2.1.47.1.1.1.1.11",
    description="The serial number of the physical entity.",
    value_type="str",
    value_mapping=None,
)


#  -------- stack -------- #

#   # -------Huawei stack------- #

hwStackRun = SnmpItem(
    name="hwStackRun",
    oid=".1.3.6.1.4.1.2011.5.25.183.1.1",
    description="The running status of the Huawei stack.",
    value_type="int",
    value_mapping=HW_STACK_RUN_MAPPING,
)
hwMemberCurrentStackId = SnmpItem(
    name="hwMemberCurrentStackId",
    oid=".1.3.6.1.4.1.2011.5.25.183.1.20.1.1",
    description="The current stack ID of the Huawei stack member.",
    value_type="int",  # 0-8
    value_mapping=None,
)
hwMemberStackPriority = SnmpItem(
    name="hwMemberStackPriority",
    oid=".1.3.6.1.4.1.2011.5.25.183.1.20.1.2",
    description="The priority of the Huawei stack member.",
    value_type="int",  # 1-255
    value_mapping=None,
)
hwMemberStackRole = SnmpItem(
    name="hwMemberStackRole",
    oid=".1.3.6.1.4.1.2011.5.25.183.1.20.1.3",
    description="The role of the Huawei stack member.",
    value_type="int",  # 1-4
    value_mapping=HW_STACK_ROLE_MAPPING,
)
hwMemberStackMacAddress = SnmpItem(
    name="hwMemberStackMacAddress",
    oid=".1.3.6.1.4.1.2011.5.25.183.1.20.1.4",
    description="The MAC address of the Huawei stack member.",
    value_type="str",
    value_mapping=None,
)
hwMemberStackDeviceType = SnmpItem(
    name="hwMemberStackDeviceType",
    oid=".1.3.6.1.4.1.2011.5.25.183.1.20.1.5",
    description="The device type of the Huawei stack member.",
    value_type="str",
    value_mapping=None,
)

#   # -------Huawei CSS stack------- #

hwCssEnable = SnmpItem(
    name="hwCssEnable",
    oid=".1.3.6.1.4.1.2011.5.25.183.1.1",
    description="The enable status of the Huawei CSS stack.",
    value_type="int",
    value_mapping=HW_STACK_RUN_MAPPING,
)

hwCssMemberFrameId = SnmpItem(
    name="hwCssMemberFrameId",
    oid=".1.3.6.1.4.1.2011.5.25.183.3.2.1.1",
    description="The frame ID of the Huawei CSS stack member.",
    value_type="int",
    value_mapping=None,
)
hwCssMemberRole = SnmpItem(
    name="hwCssMemberRole",
    oid=".1.3.6.1.4.1.2011.5.25.183.3.2.1.8",
    description="The role of the Huawei CSS stack member.",
    value_type="int",
    value_mapping=HW_STACK_ROLE_MAPPING,
)

hwCssMemberPriority = SnmpItem(
    name="hwCssMemberPriority",
    oid=".1.3.6.1.4.1.2011.5.25.183.3.2.1.3",
    description="The priority of the Huawei CSS stack member.",
    value_type="int",
    value_mapping=None,
)
hwCssMemberConfigPriority = SnmpItem(
    name="hwCssMemberConfigPriority",
    oid=".1.3.6.1.4.1.2011.5.25.183.3.2.1.4",
    description="The config priority of the Huawei CSS stack member.",
    value_type="int",
    value_mapping=None,
)

# -------- stack Cisco ------- #
cswSwitchRole = SnmpItem(
    name="cswSwitchRole",
    oid=".1.3.6.1.4.1.9.9.500.1.2.1.1.3",
    description="The role of the Cisco switch.",
    value_type="int",
    value_mapping=HW_STACK_ROLE_MAPPING,
)

cswSwitchHwPriority = SnmpItem(
    name="cswSwitchHwPriority",
    oid=".1.3.6.1.4.1.9.9.500.1.2.1.1.5",
    description="The hardware priority of the Cisco switch.",
    value_type="int",
    value_mapping=None,
)
cswSwitchState = SnmpItem(
    name="cswSwitchState",
    oid=".1.3.6.1.4.1.9.9.500.1.2.1.1.6",
    description="The state of the Cisco switch.",
    value_type="int",
    value_mapping=HW_STACK_RUN_MAPPING,
)

cswSwitchMacAddress = SnmpItem(
    name="cswSwitchMacAddress",
    oid=".1.3.6.1.4.1.9.9.500.1.2.1.1.7",
    description="The MAC address of the Cisco switch.",
    value_type="str",
    value_mapping=None,
)

# --------- stack H3C --------
h3cStackMemberID = SnmpItem(
    name="h3cStackMemberID",
    oid=".1.3.6.1.4.1.2011.10.2.91.2.1.1",
    description="The ID of the H3C stack member.",
    value_type="int",
    value_mapping=None,
)
h3cStackConfigMemberID = SnmpItem(
    name="h3cStackConfigMemberID",
    oid=".1.3.6.1.4.1.2011.10.2.91.2.1.2",
    description="The config ID of the H3C stack member.",
    value_type="int",
    value_mapping=None,
)
h3cStackPriority = SnmpItem(
    name="h3cStackPriority",
    oid=".1.3.6.1.4.1.2011.10.2.91.2.1.3",
    description="The priority of the H3C stack member.",
    value_type="int",
    value_mapping=None,
)
# ---------- RuiJie stack ----------
scMemberMacAddress = SnmpItem(
    name="scMemberMacAddress",
    oid=".1.3.6.1.4.1.4881.1.1.10.2.31.1.2.1.1.1",
    description="The MAC address of the RuiJie stack member.",
    value_type="str",
    value_mapping=None,
    to_hex=True,
)
scMemberNumber = SnmpItem(
    name="scMemberNumber",
    oid=".1.3.6.1.4.1.4881.1.1.10.2.31.1.2.1.1.2",
    description="The number of the RuiJie stack member.",
    value_type="int",
    value_mapping=None,
)
scMemberOperStatus = SnmpItem(
    name="scMemberOperStatus",
    oid=".1.3.6.1.4.1.4881.1.1.10.2.31.1.2.1.1.3",
    description="The oper status of the RuiJie stack member.",
    value_type="int",
    value_mapping=HW_STACK_RUN_MAPPING,
)
scMemberDeviceID = SnmpItem(
    name="scMemberDeviceID",
    oid=".1.3.6.1.4.1.4881.1.1.10.2.31.1.2.1.1.4",
    description="The value of mac address of neighbor lldp pdu.",
    value_type="str",
    value_mapping=None,
    to_hex=True,
)
scMemberRowStatus = SnmpItem(
    name="scMemberRowStatus",
    oid=".1.3.6.1.4.1.4881.1.1.10.2.31.1.2.1.1.5",
    description="The row status of the RuiJie stack member.",
    value_type="int",
    value_mapping=None,
)

panSysSwVersion = SnmpItem(
    name="panSysSwVersion",
    oid=".1.3.6.1.4.1.25461.2.1.2.1.1",
    description="The version of the PAN-OS software.",
    value_type="str",
    value_mapping=None,
)
panSysSerialNumber = SnmpItem(
    name="panSysSerialNumber",
    oid=".1.3.6.1.4.1.25461.2.1.2.1.3",
    description="The serial number of the PAN-OS.",
    value_type="str",
    value_mapping=None,
)

sysExtSwitchSWVersion = SnmpItem(
    name="sysExtSwitchSWVersion",
    oid=".1.3.6.1.4.1.14823.2.2.1.2.1.19.1.4",
    description="The version of the external switch software.",
    value_type="str",
    value_mapping=None,
)

sysExtSwitchSetNo = SnmpItem(
    name="sysExtSwitchSetNo",
    oid=".1.3.6.1.4.1.14823.2.2.1.2.1.19.1.7",
    description="The serial number of the external switch.",
    value_type="str",
    value_mapping=None,
)



from ipaddress import IPv4Network, IPv6Network
from typing import Literal, NamedTuple, TypeAlias, TypedDict

IPvANyNetwork: TypeAlias = IPv4Network | IPv6Network  # noqa: UP040

DiscoveryItem: TypeAlias = Literal[  # noqa: UP040
    "hostname",
    "sys_descr",
    "chassis_id",
    "uptime",
    "interfaces",
    "lldp_neighbors",
    "stack",
    "vlans",
    "prefixes",
    "routes",
    "entities",
    "mac_address_table",
    "arp_table",
]

DispatchItem: TypeAlias = Literal["sys_object_id"]  # noqa: UP040


class DeviceType(TypedDict):
    platform: str
    manufacturer: str
    model: str


class SnmpDiscovery(NamedTuple):
    name: str
    oid: str
    snmp_index: str
    snmp_value: str | float | int


class SnmpItem(NamedTuple):
    name: str
    oid: str
    value_type: Literal["str", "int", "float"]
    description: str | None = None
    value_mapping: dict[int, str] | None = None
    to_hex: bool = False

    def get_value_type(self) -> str | int | float:
        type_mapping = {"str": str, "int": int, "float": float}
        return type_mapping[self.value_type]


class Interface(TypedDict):
    if_index: int
    if_name: str
    if_descr: str
    if_type: int
    if_mtu: int
    if_speed: int
    if_high_speed: int
    if_phys_address: str
    if_admin_status: str
    if_oper_status: str
    if_ip_address: list[str]
    if_port_mode: str | None


class LldpNeighbor(TypedDict):
    local_chassis_id: str
    local_hostname: str
    local_if_name: str
    local_if_descr: str
    remote_chassis_id: str
    remote_hostname_id: str
    remote_if_name: str
    remote_if_descr: str


class Entity(TypedDict):
    ent_physical_class: int
    ent_physical_descr: str
    ent_physical_name: str
    ent_physical_software_rev: str
    ent_physical_serial_num: str


class StackMember(TypedDict):
    id: int
    priority: int
    role: str
    mac_address: str


class DiscoveryException(TypedDict):
    item: DispatchItem | DiscoveryItem
    exception: str


class SnmpDiscoveryData(TypedDict, total=False):
    hostname: str | None
    sys_descr: str | None
    uptime: str | None
    chassis_id: str | None
    interfaces: list[Interface]
    lldp_neighbors: list[LldpNeighbor]
    stack: list[dict]
    vlans: list[dict]
    prefixes: list[dict] | None
    routes: list[dict] | None
    exceptions: list[DiscoveryException]


class DiscoveryData(SnmpDiscoveryData):
    device_type: str
    manufacturer: str
    platform: str


class DiscoveryResponse(TypedDict):
    ip: str
    data: DiscoveryData | None
    snmp_reachable: bool
    icmp_reachable: bool
    ssh_reachable: bool
    sys_object_id: str | None
    exceptions: list[DiscoveryException] | None



from collections import defaultdict
from ipaddress import ip_interface
from typing import Any, Literal, TypedDict

from ezsnmp import EzSNMPError, Session

from netty_snmp._types import DiscoveryException, DiscoveryItem, Entity, Interface, LldpNeighbor, SnmpDiscoveryData
from netty_snmp.factory import consts
from netty_snmp.utils import mac_address_validator


class SnmpVersionError(Exception):
    pass


class SnmpConnectionError(Exception):
    pass


class SnmpV3Params(TypedDict):
    context_engine_id: str
    security_username: str
    security_level: Literal["no_auth_or_privacy", "auth_without_privacy", "auth_with_privacy"]
    auth_protocol: Literal["md5", "sha1", "sha224", "sha256", "sha384", "sha512"]
    auth_password: str
    privacy_protocol: Literal["des", "aes128", "aes192", "aes256"]
    privacy_password: str


class SnmpFactory:
    def __init__(
        self,
        ip: str,
        port: int = consts.SNMP_DEFAULT_PORT,
        version: consts.SnmpVersion = consts.SnmpVersion.v2c,
        community: str | None = consts.SNMP_DEFAULT_COMMUNITY,
        v3_params: SnmpV3Params | None = None,
        snmp_max_repetitions: int = consts.SNMP_MAX_repetitions,
    ) -> None:
        """
        :param ip: network device ip, ipv4 or ipv6 address
        :param port: network device snmp agent port, default: 161
        :param version: SNMP version, default: v2c
        :param community: SNMP v2 community, default: public
        :param v3_params: SNMP v3 params
        :param snmp_max_repetitions: SNMP max repetitions for bulkwalk
        """
        self.ip = ip
        self.port = port
        self.version = version
        self.community = community
        self.v3_params = v3_params
        self.session = self._session()
        self.exceptions: list[DiscoveryException] = []
        self.snmp_max_repetitions = snmp_max_repetitions

    def _session(self) -> Session:
        """create session for snmp query"""
        if self.version == consts.SnmpVersion.v2c and self.community:
            return Session(
                hostname=self.ip,
                remote_port=self.port,
                community=self.community,
                version=consts.SnmpVersion.v2c.value,
                use_long_names=False,
                use_enums=False,
                use_sprint_value=True,
            )
        if self.version == consts.SnmpVersion.v3 and self.v3_params:
            return Session(
                hostname=self.ip,
                remote_port=self.port,
                version=consts.SnmpVersion.v3.value,
                **self.v3_params,
                use_long_names=False,
                use_enums=False,
                use_sprint_value=True,
            )
        raise SnmpVersionError(f"Unsupported SNMP version: {self.version}")

    @property
    def hostname(self) -> str | None:
        """
        collect network device hostname
        """
        try:
            return self.session.get(consts.sysName.oid).value
        except EzSNMPError as e:
            self.exceptions.append(DiscoveryException(item="hostname", exception=str(e)))
            return None

    @property
    def sys_descr(self) -> str | None:
        """
        collect network device system description, include model/version/patch information
        without structured data format
        """
        try:
            return self.session.get(consts.sysDescr.oid).value
        except EzSNMPError as e:
            self.exceptions.append(DiscoveryException(item="sys_descr", exception=str(e)))
            return None

    @property
    def uptime(self) -> str | None:
        """
        collect network device uptime`
        """
        try:
            return self.session.get(consts.sysUpTime.oid).value
        except EzSNMPError as e:
            self.exceptions.append(DiscoveryException(item="uptime", exception=str(e)))
            return None

    @property
    def chassis_id(self) -> str | None:
        """
        collect chassis id via `LLDP-MIB`.
        Special configuration for Huawei: snmp should include iso view and mib-2
        """
        try:
            return mac_address_validator(self.session.get(consts.lldpLocChassisId.oid).value)
        except EzSNMPError as e:
            self.exceptions.append(DiscoveryException(item="chassis_id", exception=str(e)))
            return None

    @property
    def if_port_mode(self) -> dict[str, str]:
        """
        collect interface port mode, need implement in manufacturer factory
        """
        return {}

    @property
    def interfaces(self) -> list[Interface]:
        """
        collect interfaces via `IF-MIB`, if filtering by the items, implement it the manufacturer factory
        """
        try:
            if_index = self.session.bulkwalk(consts.ifIndex.oid, max_repetitions=self.snmp_max_repetitions)
            if_name = self.session.bulkwalk(consts.ifDescr.oid, max_repetitions=self.snmp_max_repetitions)
            if_descr = self.session.bulkwalk(consts.ifAlias.oid, max_repetitions=self.snmp_max_repetitions)
            if_mtu = self.session.bulkwalk(consts.ifMtu.oid, max_repetitions=self.snmp_max_repetitions)
            if_speed = self.session.bulkwalk(consts.ifSpeed.oid, max_repetitions=self.snmp_max_repetitions)
            if_high_speed = self.session.bulkwalk(consts.ifHighSpeed.oid, max_repetitions=self.snmp_max_repetitions)
            if_type = self.session.bulkwalk(consts.ifType.oid, max_repetitions=self.snmp_max_repetitions)
            if_phys_addr = self.session.bulkwalk(consts.ifPhysAddr.oid, max_repetitions=self.snmp_max_repetitions)
            if_admin = self.session.bulkwalk(consts.ifAdminStatus.oid, max_repetitions=self.snmp_max_repetitions)
            if_oper = self.session.bulkwalk(consts.ifOperStatus.oid, max_repetitions=self.snmp_max_repetitions)
            if_addr_index = self.session.bulkwalk(consts.ifAdEntIfIndex.oid, max_repetitions=self.snmp_max_repetitions)
            if_addr_netmask = self.session.bulkwalk(
                consts.ifAdEntNetMask.oid, max_repetitions=self.snmp_max_repetitions
            )
            if_port_mode = self.if_port_mode
        except EzSNMPError as e:
            self.exceptions.append(DiscoveryException(item="interfaces", exception=str(e)))
            return []
        index_if_index = {x.oid_index: x.value for x in if_index}
        index_if_name = {x.oid_index: x.value for x in if_name}
        index_if_descr = {x.oid_index: x.value for x in if_descr}
        index_if_mtu = {x.oid_index: x.value for x in if_mtu}
        index_if_speed = {x.oid_index: x.value for x in if_speed}
        index_if_high_speed = {x.oid_index: x.value for x in if_high_speed}
        index_if_type = {x.oid_index: x.value for x in if_type}
        index_if_phys_addr = {x.oid_index: x.value for x in if_phys_addr}
        index_if_admin = {x.oid_index: x.value for x in if_admin}
        index_if_oper = {x.oid_index: x.value for x in if_oper}
        index_if_addr_index = defaultdict(list)
        index_if_addr_netmask = {x.oid_index: x.value for x in if_addr_netmask}
        for x in if_addr_index:
            netmask = index_if_addr_netmask[x.oid_index]
            index_if_addr_index[x.value].append(ip_interface(f"{x.oid_index}/{netmask}"))
        return [
            Interface(
                if_index=int(x),
                if_name=index_if_name.get(x),
                if_descr=index_if_descr.get(x),
                if_mtu=int(index_if_mtu.get(x)),
                if_speed=int(index_if_speed.get(x)),
                if_high_speed=int(index_if_high_speed.get(x)),
                if_type=index_if_type.get(x),
                if_phys_address=mac_address_validator(index_if_phys_addr.get(x)),
                if_admin_status=index_if_admin.get(x),
                if_oper_status=index_if_oper.get(x),
                if_ip_address=index_if_addr_index.get(x),
                if_port_mode=if_port_mode.get(x, consts.UNKNOWN_PORT_MODE),
            )
            for x in index_if_index
            if x is not None
        ]

    @property
    def lldp_neighbors(self) -> list[LldpNeighbor]:
        """
        collect lldp neighbors via `LLDP-MIB`
        Special configuration for network devices: include iso mib-2 view in snmp configuration
        lldpRemChassisID: snmp_index value the last of 2 is local portID's index(last of 1)
        """
        try:
            local_chassis_id = self.chassis_id
            local_if_name = self.session.bulkwalk(consts.lldpLoPortId.oid, max_repetitions=self.snmp_max_repetitions)
            local_if_descr = self.session.bulkwalk(
                consts.lldpLocPortDesc.oid, max_repetitions=self.snmp_max_repetitions
            )
            remote_chassis_id = self.session.bulkwalk(
                consts.lldpRemChassisId.oid, max_repetitions=self.snmp_max_repetitions
            )
            remote_hostname = self.session.bulkwalk(
                consts.lldpRemSysName.oid, max_repetitions=self.snmp_max_repetitions
            )
            remote_if_name = self.session.bulkwalk(consts.lldpRemPortId.oid, max_repetitions=self.snmp_max_repetitions)
            remote_if_descr = self.session.bulkwalk(
                consts.lldpRemPortDesc.oid, max_repetitions=self.snmp_max_repetitions
            )
        except EzSNMPError as e:
            self.exceptions.append(DiscoveryException(item="lldp_neighbors", exception=str(e)))
            return []
        index_local_if_name = {x.oid.split(".")[-1]: x.value for x in local_if_name}
        index_local_if_descr = {x.oid.split(".")[-1]: x.value for x in local_if_descr}
        index_remote_chassis_id = {x.oid.split(".")[-2]: mac_address_validator(x.value) for x in remote_chassis_id}
        index_remote_hostname = {x.oid.split(".")[-2]: x.value for x in remote_hostname}
        index_remote_if_name = {x.oid.split(".")[-2]: x.value for x in remote_if_name}
        index_remote_if_descr = {x.oid.split(".")[-2]: x.value for x in remote_if_descr}
        return [
            LldpNeighbor(
                local_chassis_id=local_chassis_id,
                local_hostname=self.hostname,
                local_if_name=index_local_if_name.get(x),
                local_if_descr=index_local_if_descr.get(x),
                remote_chassis_id=index_remote_chassis_id.get(x),
                remote_hostname_id=index_remote_hostname.get(x),
                remote_if_name=index_remote_if_name.get(x),
                remote_if_descr=index_remote_if_descr.get(x),
            )
            for x in index_remote_chassis_id
            if x is not None
        ]

    @property
    def entities(self) -> list[Entity]:
        """
        collect entities via `ENTITY-MIB`
        basically: chassis(3) should be the main module of device.
        but fuck huawei (3 or 9(module) for different product lines) because its unstandard implementation
        see value mapping in `consts.py`: ENTITY_PHYSICAL_CLASS_MAPPING
        so it may won't work for huawei in default factory.
        """
        try:
            ent_phy_class = self.session.bulkwalk(consts.entPhysicalClass.oid)
            index_ent_phy_class = {x.oid.split(".")[-1]: x.value for x in ent_phy_class if int(x.value) == 3}  # noqa: PLR2004
            oids = list(index_ent_phy_class.keys())
            if not oids:
                return []
            ent_phy_descr = self.session.get([consts.entPhysicalDescr.oid + "." + x for x in oids])
            ent_phy_name = self.session.get([consts.entPhysicalName.oid + "." + x for x in oids])
            ent_phy_software = self.session.get([consts.entPhysicalSoftwareRev.oid + "." + x for x in oids])
            ent_phy_serial = self.session.get([consts.entPhysicalSerialNum.oid + "." + x for x in oids])
        except EzSNMPError as e:
            self.exceptions.append(DiscoveryException(item="entities", exception=str(e)))
            return []
        index_ent_phy_descr = {x.oid.split(".")[-1]: x.value for x in ent_phy_descr}
        index_ent_phy_name = {x.oid.split(".")[-1]: x.value for x in ent_phy_name}
        index_ent_phy_software = {x.oid.split(".")[-1]: x.value for x in ent_phy_software}
        index_ent_phy_serial = {x.oid.split(".")[-1]: x.value for x in ent_phy_serial}
        return [
            Entity(
                ent_physical_class=int(x),
                ent_physical_descr=index_ent_phy_descr.get(x),
                ent_physical_name=index_ent_phy_name.get(x),
                ent_physical_software_rev=index_ent_phy_software.get(x),
                ent_physical_serial_num=index_ent_phy_serial.get(x),
            )
            for x in index_ent_phy_class
            if x is not None
        ]

    @property
    def stack(self) -> Any: ...

    @property
    def vlans(self) -> Any: ...

    @property
    def prefixes(self) -> Any: ...

    @property
    def routes(self) -> Any: ...

    @property
    def mac_address_table(self) -> Any: ...

    @property
    def arp_table(self) -> Any: ...

    def discovery(self, items: list[DiscoveryItem] | None = None) -> SnmpDiscoveryData:
        """
        discovery network basic information via SNMP
        params:
            items: list of `DiscoveryItem` to collect, if not specified, all items will be collected
        returns:
            SnmpDiscoveryData: all collected items
        """
        if not items:
            return SnmpDiscoveryData(
                hostname=self.hostname,
                sys_descr=self.sys_descr,
                uptime=self.uptime,
                chassis_id=self.chassis_id,
                interfaces=self.interfaces,
                lldp_neighbors=self.lldp_neighbors,
                stack=self.stack,
                vlans=self.vlans,
                prefixes=self.prefixes,
                routes=self.routes,
                exceptions=self.exceptions,
            )
        return {x: getattr(self, x) for x in items}  # type: ignore  # noqa: PGH003



from enum import StrEnum

from netty_snmp._types import DeviceType
from netty_snmp.device_type.manufactures.arista import ARISTA_DEVICE_TYPES
from netty_snmp.device_type.manufactures.aruba import ARUBA_DEVICE_TYPES
from netty_snmp.device_type.manufactures.cisco import CISCO_DEVICE_TYPES
from netty_snmp.device_type.manufactures.fortinet import FORTINET_DEVICE_TYPES
from netty_snmp.device_type.manufactures.h3c import H3C_DEVICE_TYPES
from netty_snmp.device_type.manufactures.huawei import HUAWEI_DEVICE_TYPES
from netty_snmp.device_type.manufactures.juniper import JUNIPER_DEVICE_TYPES
from netty_snmp.device_type.manufactures.paloalto import PALOALTO_DEVICE_TYPES
from netty_snmp.device_type.manufactures.ruijie import RUIJIE_DEVICE_TYPES
from netty_snmp.factory.consts import UNKNOWN_MODEL, UNKNOWN_PLATFORM


class Manufacturer(StrEnum):
    Cisco = "Cisco"
    Huawei = "Huawei"
    Aruba = "Aruba"
    Arista = "Arista"
    RuiJie = "Ruijie"
    H3C = "H3C"
    PaloAlto = "Palo Alto"
    FortiNet = "FortiNet"
    Juniper = "Juniper"
    Netgear = "Netgear"
    TPLink = "TP-Link"
    Ruckus = "Ruckus"
    CheckPoint = "CheckPoint"
    Sangfor = "Sangfor"


class Platform(StrEnum):
    CiscoIOS = "cisco_ios"
    CiscoIOSXE = "cisco_xe"
    CiscoIOSXR = "cisco_xr"
    CiscoNexusOS = "cisco_nxos"
    Huawei = "huawei"
    HuaweiVRP = "huawei_vrp"
    HuaweiVRPV8 = "huawei_vrpv8"
    Aruba = "aruba_os"
    ArubaOSSwitch = "aruba_osswitch"
    Arista = "arista_eos"
    RuiJie = "ruijie_os"
    H3C = "hp_comware"
    FortiNet = "fortinet"
    PaloAlto = "paloalto_panos"
    Juniper = "juniper_junos"
    Netgear = "netgear_prosafe"
    TPLink = "tplink_jetstream"
    Ruckus = "ruckus_fastiron"
    Sangfor = "Unknown"


EnterpriseIdManufacturer: dict[str, Manufacturer] = {
    "2011": Manufacturer.Huawei,
    "9": Manufacturer.Cisco,
    "14823": Manufacturer.Aruba,
    "30065": Manufacturer.Arista,
    "4881": Manufacturer.RuiJie,
    "61878": Manufacturer.H3C,
    "25461": Manufacturer.PaloAlto,
    "12356": Manufacturer.FortiNet,
    "2636": Manufacturer.Juniper,
    "4562": Manufacturer.Netgear,
    "11863": Manufacturer.TPLink,
    "25053": Manufacturer.Ruckus,
    "2620": Manufacturer.CheckPoint,
    "30547": Manufacturer.Sangfor,
}


ManufacturerDeviceTypes = {
    Manufacturer.Cisco: CISCO_DEVICE_TYPES,
    Manufacturer.Huawei: HUAWEI_DEVICE_TYPES,
    Manufacturer.Aruba: ARUBA_DEVICE_TYPES,
    Manufacturer.Arista: ARISTA_DEVICE_TYPES,
    Manufacturer.RuiJie: RUIJIE_DEVICE_TYPES,
    Manufacturer.H3C: H3C_DEVICE_TYPES,
    Manufacturer.PaloAlto: PALOALTO_DEVICE_TYPES,
    Manufacturer.FortiNet: FORTINET_DEVICE_TYPES,
    Manufacturer.Juniper: JUNIPER_DEVICE_TYPES,
}

ManufacturerDefaultPlatform = {
    Manufacturer.Cisco: Platform.CiscoIOS,
    Manufacturer.Huawei: Platform.Huawei,
    Manufacturer.Aruba: Platform.Aruba,
    Manufacturer.Arista: Platform.Arista,
    Manufacturer.RuiJie: Platform.RuiJie,
    Manufacturer.H3C: Platform.H3C,
    Manufacturer.PaloAlto: Platform.PaloAlto,
    Manufacturer.FortiNet: Platform.FortiNet,
    Manufacturer.Juniper: Platform.Juniper,
    Manufacturer.Netgear: Platform.Netgear,
    Manufacturer.TPLink: Platform.TPLink,
    Manufacturer.Ruckus: Platform.Ruckus,
}


def _forecast_platform(device_type: DeviceType) -> Platform:
    if device_type.get("platform"):
        return device_type["platform"]  # type: ignore  # noqa: PGH003
    # Optimization: add other platforms forecasting according to model name and vendor
    if device_type["manufacturer"] == Manufacturer.Cisco:
        if device_type["model"].startswith(("C9", "C38")):
            device_type["platform"] = Platform.CiscoIOSXE
        elif device_type["model"].startswith("N"):
            device_type["platform"] = Platform.CiscoNexusOS
        elif device_type["model"].lower().startswith("isr"):
            device_type["platform"] = Platform.CiscoIOSXR
        else:
            device_type["platform"] = Platform.CiscoIOS
    if device_type["manufacturer"] == Manufacturer.Huawei:
        if device_type["model"].startswith("CE") or device_type["model"].startswith("FM"):
            device_type["platform"] = Platform.HuaweiVRPV8
        else:
            device_type["platform"] = Platform.HuaweiVRP
    device_type["platform"] = ManufacturerDefaultPlatform[device_type["manufacturer"]]  # type: ignore  # noqa: PGH003

    return device_type["platform"]


def _get_manufacturer_platform(device_type: DeviceType) -> DeviceType:
    if not device_type.get("platform"):
        device_type["platform"] = _forecast_platform(device_type)
    return device_type


def get_device_type(sys_object_id: str) -> DeviceType | None:
    private_enterprise_id = sys_object_id.split(".1.3.6.1.4.1.")[1].split(".")[0]
    if private_enterprise_id not in EnterpriseIdManufacturer:
        return None

    device_type = ManufacturerDeviceTypes[EnterpriseIdManufacturer[private_enterprise_id]].get(sys_object_id)
    if not device_type:
        device_type = DeviceType(
            manufacturer=EnterpriseIdManufacturer[private_enterprise_id],
            model=UNKNOWN_MODEL,
            platform=UNKNOWN_PLATFORM,
        )
    return _get_manufacturer_platform(device_type)


def strings_to_dict(strings: str, manufacturer: Manufacturer, platform: Platform) -> dict:
    # basic sysbojectids data source:
    # 1. https://bestmonitoringtools.com/identify-devices-with-snmp-system-oid-sysobjectid-database/
    # 2.
    result = {}
    d1 = [line.split("\t") for line in strings.splitlines() if line]
    for x in d1:
        if len(x) < 3:  # noqa: PLR2004
            continue
        result[x[0]] = {
            "manufacturer": manufacturer,
            "platform": platform,
            "model": x[2],
        }
    return result



from typing import TYPE_CHECKING

if TYPE_CHECKING:
    from netty_snmp._types import DeviceType


TP_LINK_DEVICE_TYPES: dict[str, "DeviceType"] = {
    "1.3.6.1.4.1.11863.1.1.35": {"manufacturer": "TP-Link", "platform": "tplink_jetstream", "model": "T1500-28PCT"},
    "1.3.6.1.4.1.11863.3.2.10": {"manufacturer": "TP-Link", "platform": "tplink_jetstream", "model": "EAP245"},
    "1.3.6.1.4.1.11863.5.29": {"manufacturer": "TP-Link", "platform": "tplink_jetstream", "model": "T1600G-52TS"},
}
